==> cdo_1.9.6~rc4.orig.tar.xz: cdo-1.9.6~rc4/src/Deltime.cc <==
/*
  This file is part of CDO. CDO is a collection of Operators to
  manipulate and analyse Climate model Data.

  Copyright (C) 2003-2018 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
  See COPYING file for copying and redistribution conditions.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; version 2 of the License.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/

#include <cdi.h>

#include "cdo_int.h"
#include "pstream_int.h"
#include "util_string.h"

void *
Deltime(void *process)
{
  int nrecs;
  int varID, levelID;
  bool copytimestep;
  size_t nmiss;
  int year, month, day;
  int dday, dmon;
  const char *cmons[] = { "", "jan", "feb", "mar", "apr", "may", "jun", "jul", "aug", "sep", "oct", "nov", "dec" };

  cdoInitialize(process);

  bool lcopy = UNCHANGED_RECORD;

  // clang-format off
  int DELDAY   = cdoOperatorAdd("delday",   0, 0, NULL);
  int DEL29FEB = cdoOperatorAdd("del29feb", 0, 0, NULL);
  // clang-format on

  UNUSED(DELDAY);

  int operatorID = cdoOperatorID();

  if (operatorID == DEL29FEB)
    {
      dday = 29;
      dmon = 2;
    }
  else
    {
      int nsel = operatorArgc();
      if (nsel < 1) cdoAbort("Too few arguments!");
      if (nsel > 1) cdoAbort("Too many arguments!");
      char *sarg = operatorArgv()[0];
      dday = atoi(sarg);
      dmon = 0;
      while (isdigit(*sarg)) sarg++;
      if (isalpha(*sarg))
        {
          char smon[32];
          strncpy(smon, sarg, sizeof(smon) - 1);
          smon[sizeof(smon) - 1] = 0;
          strtolower(smon);
          int im;
          for (im = 0; im < 12; ++im)
            if (memcmp(smon, cmons[im + 1], 3) == 0) break;

          if (im < 12) dmon = im + 1;
        }
    }

  if (Options::cdoVerbose) cdoPrint("delete day %d%s", dday, cmons[dmon]);

  int streamID1 = cdoStreamOpenRead(cdoStreamName(0));

  int vlistID1 = cdoStreamInqVlist(streamID1);
  int vlistID2 = vlistDuplicate(vlistID1);

  int taxisID1 = vlistInqTaxis(vlistID1);
  int taxisID2 = taxisDuplicate(taxisID1);
  taxisDefCalendar(taxisID2, CALENDAR_365DAYS);
  vlistDefTaxis(vlistID2, taxisID2);

  int streamID2 = cdoStreamOpenWrite(cdoStreamName(1), cdoFiletype());
  pstreamDefVlist(streamID2, vlistID2);

  std::vector<double> array;
  if (!lcopy)
    {
      size_t gridsizemax = vlistGridsizeMax(vlistID1);
      array.resize(gridsizemax);
    }

  int nfound = 0;
  int tsID = 0;
  int tsID2 = 0;
  while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
    {
      int64_t vdate = taxisInqVdate(taxisID1);
      // vtime = taxisInqVtime(taxisID1);

      cdiDecodeDate(vdate, &year, &month, &day);

      if (day == dday && (month == dmon || dmon == 0))
        {
          nfound++;
          copytimestep = false;
          if (Options::cdoVerbose) cdoPrint("Delete %4.4d-%2.2d-%2.2d at timestep %d", year, month, day, tsID + 1);
        }
      else
        copytimestep = true;

      if (copytimestep)
        {
          taxisCopyTimestep(taxisID2, taxisID1);
          pstreamDefTimestep(streamID2, tsID2++);

          for (int recID = 0; recID < nrecs; recID++)
            {
              pstreamInqRecord(streamID1, &varID, &levelID);
              pstreamDefRecord(streamID2, varID, levelID);
              if (lcopy)
                {
                  pstreamCopyRecord(streamID2, streamID1);
                }
              else
                {
                  pstreamReadRecord(streamID1, array.data(), &nmiss);
                  pstreamWriteRecord(streamID2, array.data(), nmiss);
                }
            }
        }

      tsID++;
    }

  cdoStreamClose(streamID2);
  cdoStreamClose(streamID1);

  if (nfound == 0) cdoWarning("Day %d%s not found!", dday, cmons[dmon]);

  vlistDestroy(vlistID2);

  cdoFinish();

  return 0;
}

==> cdo_1.9.7.1.orig.tar.xz: cdo-1.9.7.1/src/Deltime.cc <==
/*
  This file is part of CDO. CDO is a collection of Operators to
  manipulate and analyse Climate model Data.

  Copyright (C) 2003-2019 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
  See COPYING file for copying and redistribution conditions.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; version 2 of the License.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/

#include <cdi.h>

#include "cdo_int.h"

#include "util_string.h"

void *
Deltime(void *process)
{
  int nrecs;
  int varID, levelID;
  bool copytimestep;
  size_t nmiss;
  int year, month, day;
  int dday, dmon;
  const char *cmons[] = { "", "jan", "feb", "mar", "apr", "may", "jun", "jul", "aug", "sep", "oct", "nov", "dec" };

  cdoInitialize(process);

  bool lcopy = unchangedRecord();

  // clang-format off
  int DELDAY   = cdoOperatorAdd("delday",   0, 0, nullptr);
  int DEL29FEB = cdoOperatorAdd("del29feb", 0, 0, nullptr);
  // clang-format on

  UNUSED(DELDAY);

  int operatorID = cdoOperatorID();

  if (operatorID == DEL29FEB)
    {
      dday = 29;
      dmon = 2;
    }
  else
    {
      int nsel = operatorArgc();
      if (nsel < 1) cdoAbort("Too few arguments!");
      if (nsel > 1) cdoAbort("Too many arguments!");
      char *sarg = operatorArgv()[0];
      dday = atoi(sarg);
      dmon = 0;
      while (isdigit(*sarg)) sarg++;
      if (isalpha(*sarg))
        {
          char smon[32];
          strncpy(smon, sarg, sizeof(smon) - 1);
          smon[sizeof(smon) - 1] = 0;
          cstrToLowerCase(smon);
          int im;
          for (im = 0; im < 12; ++im)
            if (memcmp(smon, cmons[im + 1], 3) == 0) break;

          if (im < 12) dmon = im + 1;
        }
    }

  if (Options::cdoVerbose) cdoPrint("delete day %d%s", dday, cmons[dmon]);

  CdoStreamID streamID1 = cdoOpenRead(0);

  int vlistID1 = cdoStreamInqVlist(streamID1);
  int vlistID2 = vlistDuplicate(vlistID1);

  int taxisID1 = vlistInqTaxis(vlistID1);
  int taxisID2 = taxisDuplicate(taxisID1);
  taxisDefCalendar(taxisID2, CALENDAR_365DAYS);
  vlistDefTaxis(vlistID2, taxisID2);

  CdoStreamID streamID2 = cdoOpenWrite(1);
  cdoDefVlist(streamID2, vlistID2);

  std::vector<double> array;
  if (!lcopy)
    {
      size_t gridsizemax = vlistGridsizeMax(vlistID1);
      array.resize(gridsizemax);
    }

  int nfound = 0;
  int tsID = 0;
  int tsID2 = 0;
  while ((nrecs = cdoStreamInqTimestep(streamID1, tsID)))
    {
      int64_t vdate = taxisInqVdate(taxisID1);
      // vtime = taxisInqVtime(taxisID1);

      cdiDecodeDate(vdate, &year, &month, &day);

      if (day == dday && (month == dmon || dmon == 0))
        {
          nfound++;
          copytimestep = false;
          if (Options::cdoVerbose) cdoPrint("Delete %4.4d-%2.2d-%2.2d at timestep %d", year, month, day, tsID + 1);
        }
      else
        copytimestep = true;

      if (copytimestep)
        {
          taxisCopyTimestep(taxisID2, taxisID1);
          cdoDefTimestep(streamID2, tsID2++);

          for (int recID = 0; recID < nrecs; recID++)
            {
              cdoInqRecord(streamID1, &varID, &levelID);
              cdoDefRecord(streamID2, varID, levelID);
              if (lcopy)
                {
                  cdoCopyRecord(streamID2, streamID1);
                }
              else
                {
                  cdoReadRecord(streamID1, array.data(), &nmiss);
                  cdoWriteRecord(streamID2, array.data(), nmiss);
                }
            }
        }

      tsID++;
    }

  cdoStreamClose(streamID2);
  cdoStreamClose(streamID1);

  if (nfound == 0) cdoWarning("Day %d%s not found!", dday, cmons[dmon]);

  vlistDestroy(vlistID2);

  cdoFinish();

  return nullptr;
}
